/* Copyright (C) 2006 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#ifndef __EDITOR_H__
#define __EDITOR_H__
#include <QAction>
#include <QToolBar>
#include <QTabWidget>
#include <QDropEvent>
#include "basewidget.h"
#include "syntax.h"
#include "octave_connection.h"
#include "search_dialog.h"
#include "codeedit.h"
#include "numberedtextview.h"
#include "projects/projects.h"

class Editor: public BaseWidget
{
  Q_OBJECT
 public:
  /**New editor. If multiple==false, it returns new editor or 0 if editor instance already exists.*/
  static Editor * createEditor(QWidget *parent, bool multiple=false);
  
  /**Returns actual editor instance or 0.*/
  static Editor * editorInstance();

  void setOctaveConnection(OctaveConnection *octave_connection);
  
  /**Sets active project name.*/
  void setProject(QString name);
  
  /**Gets active project name.*/
  QString getProject();
  
  /**Load a list of files.*/
  void loadFiles(const QStringList &files);
  
  /**Close one tab. If close_all_tabs== true close all tabs. It's usefull for load projects.*/
  void closeTabs(bool close_all_tabs);
 private:
  Editor(QWidget *parent);
  ~Editor();

  //Project opened in this editor.
  QString project_name;
  
  // Pointer to the Editor instance, 0 if there is no an instance
  static Editor * instance;

  QTabWidget *tabWidget;
  QToolBar *toolBar;
  OctaveConnection *octave_connection;
  SearchDialog *search_dialog;

  NumberedTextView *currentNtv;

  // Actions
  QAction *actionNew, *actionOpen, *actionSave, *actionSaveAs, *actionPrint, *actionClose;
  QAction *actionUndo, *actionCut, *actionCopy, *actionPaste;
  QAction *actionSearch, *actionRun, *actionSendToOctave;
  QAction *actionStep;
  QAction *actionProjects;
  
 protected:
  void dropEvent ( QDropEvent * event );
  void dragEnterEvent(QDragEnterEvent *event);
  void closeEvent ( QCloseEvent * event );

 public slots:
  void toolbar_action(QAction*);
  void search();
  void replace();
  void toggleBreakpoint(int lineno);
  void tabChanged(int index);
  void textModified(bool ok=true);

  void debug(int lineno, int colno);
  void endDebug();
  
  /**Shows dialod to select or modify projects.*/
  void show_projects_dialog();
  
   /**Open file.*/
  void openFile(QString file=QString());
};

#endif
