#include "variables_list.h"
#include <QInputDialog>


VariableList::VariableList(QWidget * parent):BaseWidget(parent)
{
	setWindowIcon( QIcon( QString( ICON_PATH )+"/taskbar.png" ) );
	setWindowTitle("Variables' List");
	
	QVBoxLayout *vbox=new QVBoxLayout;
	setLayout(vbox);
	
	QHBoxLayout *hbox=new QHBoxLayout;
	vbox->addLayout(hbox);
	
	reload_button=new QPushButton(this);
	reload_button->setIcon( QIcon( QString( ICON_PATH )+"/reload.png" ) );
	reload_button->setToolTip("Reload variables' list");
	hbox->addWidget(reload_button);
	reload_button->show();
	connect(reload_button,SIGNAL(clicked ()), this, SLOT(send_whos_command_to_octave()));
	
	tree=new QTreeWidget(this);
	QStringList labels;
	labels << "Name" << "Size" << "Bytes" << "Class" /*<< "Prot"*/ ;
	tree->setHeaderLabels(labels);
	connect(tree,SIGNAL(itemDoubleClicked ( QTreeWidgetItem * , int  )), this, SLOT(item_click(QTreeWidgetItem *,int)) );
	
	vbox->addWidget(tree);
	tree->show();
	
	local_item=new QTreeWidgetItem(tree);
	local_item->setText(0,"Local user variables");
	local_item->setToolTip(0,"Local user variables");
	local_item->setIcon(0,QIcon( QString( ICON_PATH )+"/personal.png"  ) );
	QFont font(tree->font());
	font.setBold(true);
	local_item->setFont(0,font);
	currently_item=new QTreeWidgetItem(tree);
	currently_item->setText(0,"Functions");
	currently_item->setToolTip(0,"Functions");
	currently_item->setIcon(0,QIcon( QString( ICON_PATH )+"/run.png"  ) );
	currently_item->setFont(0,font);
}

void VariableList::setOctaveConnection(OctaveConnection *octave_connection)
{
	this->octave_connection=octave_connection;
	connect(octave_connection,SIGNAL(line_ready(QString)),this,SLOT(line_ready(QString)));
}

void VariableList::line_ready(QString line)
{
	if( line.startsWith("~~whos:") )
	{
		QString s(line);
		s.remove(0,7);
		if( s.trimmed().isEmpty() ) return;
		
		/*if( s.startsWith("*** dynamically") )
		{
			//text->append("<b>"+s+"</b>");
			s.remove(0,4);
			
			last_root_item=dynamically_item;
			last_root_item->setText(0,s.trimmed());
			last_root_item->setToolTip(0,s.trimmed());
			last_root_item->setIcon(0,QIcon( QString( ICON_PATH )+"/connect_established.png"  ) );
			QFont font(tree->font());
			font.setBold(true);
			last_root_item->setFont(0,font);
		}
		else*/
		if( s.startsWith("*** local") )
		{
			//text->append("<b>"+s+"</b>");
// 			s.remove(0,4);
// 			
 			last_root_item=local_item;
// 			last_root_item->setText(0,s.trimmed());
// 			last_root_item->setToolTip(0,s.trimmed());
// 			last_root_item->setIcon(0,QIcon( QString( ICON_PATH )+"/personal.png"  ) );
// 			QFont font(tree->font());
// 			font.setBold(true);
// 			last_root_item->setFont(0,font);
		}
		else if( s.startsWith("*** currently") )
		{
			//text->append("<b>"+s+"</b>");
// 			s.remove(0,4);
// 			
 			last_root_item=currently_item;
// 			last_root_item->setText(0,s.trimmed());
// 			last_root_item->setToolTip(0,s.trimmed());
// 			last_root_item->setIcon(0,QIcon( QString( ICON_PATH )+"/run.png"  ) );
// 			QFont font(tree->font());
// 			font.setBold(true);
// 			last_root_item->setFont(0,font);
		}
		else
		{
			QTreeWidgetItem *item=new QTreeWidgetItem(last_root_item);
			QStringList list = s.trimmed().split(QRegExp("\\s+"));
			int i=0;
			for(;i<list.size() && i<5;i++)
			{
				item->setText(i,list.at(i) );
				item->setToolTip(i,list.at(i) );
			}
			
			//for(i=1;i<4;i++) tree->resizeColumnToContents(i);
		}
		/*
		else
		{
			//text->append(s);
			QTreeWidgetItem *item=new QTreeWidgetItem(last_root_item);
			item->setText(0,s.trimmed());
			item->setToolTip(0,s.trimmed());
		}
		*/
	}else if (line.startsWith("~~scalar:"))
	{
	        QStringList sl = line.mid(9).split("=");
		bool ok;

		sl.last() = QInputDialog::getText(this, sl.first(), QString(),
						  QLineEdit::Normal, sl.last().trimmed(),
						  &ok);

		if(ok && !sl.last().isEmpty())
		  octave_connection->command_enter(sl.first() + " = " + sl.last());
	}else if (line.startsWith("~~string:"))
	{
	        QString s     = line.mid(9);
	        QString var   = s.section("=", 0, 0);

		s = s.section("=", 1);

		QString value = s.left(s.length() - 1);

		bool ok;

		value = QInputDialog::getText(this, var, QString(),
					      QLineEdit::Normal, value,
					      &ok);

		if(ok)
		  octave_connection->command_enter(var + " = \"" + value + "\"");
	}
}

void VariableList::windowActivated ( QWidget * /*w*/ )
{
	//send_whos_command_to_octave();
}

void VariableList::windowActivated()
{
	//send_whos_command_to_octave();
}

void VariableList::send_whos_command_to_octave()
{
	//tree->clear();
	
	for(int i=local_item->childCount()-1; i>=0; i--)
	{
		QTreeWidgetItem *item = local_item->child(i);
		local_item->removeChild(item);
		delete item;
	}
	
	for(int i=currently_item->childCount()-1; i>=0; i--)
	{
		QTreeWidgetItem *item = currently_item->child(i);
		currently_item->removeChild(item);
		delete item;
	}
	
	//local_item=new QTreeWidgetItem(tree);
	//dynamically_item=new QTreeWidgetItem(tree);
	//currently_item=new QTreeWidgetItem(tree);
	
	QString command;
	command=
	"if ( 0==0 );\n"
	"qtoctave_aux1=whos -variables;\n"
	"fprintf(stderr, \"~~whos:*** local user variables:\\n\");\n"
	"for qtoctave_aux2=1:length(qtoctave_aux1)\n"
		"fprintf(stderr, \"~~whos:%s\t%dx%d\t%d\t%s\\n\", qtoctave_aux1(qtoctave_aux2).name ,qtoctave_aux1(qtoctave_aux2).size(1),qtoctave_aux1(qtoctave_aux2).size(2), qtoctave_aux1(qtoctave_aux2).bytes, qtoctave_aux1(qtoctave_aux2).class);"
	"endfor\n"
	"qtoctave_aux1=whos -functions;\n"
	"fprintf(stderr, \"~~whos:*** currently compiled functions:\\n\");\n"
	"for qtoctave_aux2=1:length(qtoctave_aux1)\n"
		"fprintf(stderr, \"~~whos:%s\t%dx%d\t%d\t%s\\n\", qtoctave_aux1(qtoctave_aux2).name ,qtoctave_aux1(qtoctave_aux2).size(1),qtoctave_aux1(qtoctave_aux2).size(2), qtoctave_aux1(qtoctave_aux2).bytes, qtoctave_aux1(qtoctave_aux2).class);\n"
	"endfor\n"
	"clear qtoctave_aux1 qtoctave_aux2;\n"
	"endif;"
	;
	octave_connection->command_enter(command,false);
	//octave_connection->command_enter("printf(\"\\n\")",false);
}



void VariableList::item_click(QTreeWidgetItem * item, int /*column*/)
{
	QString name=item->text(0);
	QString clas=item->text(3);
	QString cmd;

	if (clas=="double" || clas=="matrix")
	{
		emit open_table(name);
	}else if (clas == "scalar")
	{
	  cmd = "fprintf(stderr, \"~~scalar:" + name + "=%g\\n\", " + name + ");";
	  octave_connection->command_enter(cmd, false);
	}else if (clas == "string")
	{
	  cmd = "fprintf(stderr, \"~~string:" + name + "=%s\\n\", " + name + ");";
	  octave_connection->command_enter(cmd, false);
	}
}

