#include "windows_layout.h"
#include <QGroupBox>
#include <QFileDialog>
#include <QMessageBox>
#include <QMdiSubWindow>

AddButton::AddButton( QWidget * parent ):QPushButton(parent)
{
	connect(this, SIGNAL(clicked()), this, SLOT(clicked_cb()) );
	widget_contained=NULL;
}

void AddButton::clicked_cb()
{
	emit button_clicked(this);
}

void AddButton::show_widgets(QMdiArea *work_space)
{
	if(widget_contained!=NULL)
	{
		widget_contained->setAttribute(Qt::WA_DeleteOnClose, false);
		work_space->removeSubWindow(widget_contained);
		widget_contained->setAttribute(Qt::WA_DeleteOnClose);
		widget_contained->setParent((QWidget*)parent());
		parent_layout->addWidget(widget_contained);
		widget_contained->show();
		parent_layout->setCurrentIndex(1);
	}
	else
	{
		((QWidget*)parent())->hide();
	}
	//hide();
}

static QWidget* build_AddButton(WindowsLayout *wl)
{
	QWidget *parent=new QWidget;
	parent->setLayout( new QStackedLayout );
	parent->layout()->setMargin(0);
	parent->show();
	
	AddButton *button=new AddButton(parent);
	QObject::connect(button, SIGNAL(button_clicked(AddButton*)), wl, SLOT(add_widget_cb(AddButton *)) );
	QObject::connect(wl, SIGNAL(show_widgets(QMdiArea*)), button, SLOT(show_widgets(QMdiArea*)) );
	button->setText("Add window");
	parent->layout()->addWidget(button);
	button->parent_layout=(QStackedLayout*)parent->layout();
	button->show();
	
	return parent;
}

WindowsLayout::WindowsLayout( QWidget * parent ):BaseWidget(parent)
{
	setWindowTitle("Windows' Layout");
	setWindowIcon( QIcon( QString( ICON_PATH )+"/run.png" ) );
	QVBoxLayout *vbox=new QVBoxLayout;
	setLayout(vbox);
	
	{
		QHBoxLayout *hbox=new QHBoxLayout;
		vbox->addLayout(hbox);
		
		QLabel *title_label=new QLabel("Title:",this);
		hbox->addWidget(title_label);
		title_label->show();
		
		title=new QLineEdit(this);
		hbox->addWidget(title);
		title->show();
		
		icon_button=new QPushButton( this);
		icon_button->setIcon( QIcon( QString( ICON_PATH )+"/run.png" ) );
		hbox->addWidget(icon_button);
		icon_button->show();
		connect(icon_button, SIGNAL(clicked () ), this, SLOT(select_icon_cb()) );
	}
	
	QSplitter *split=new QSplitter(this);
	vbox->addWidget(split);
	
	tree=new QListWidget(this);
	split->addWidget(tree);
	tree->show();
	//QListWidgetItem *item=new QListWidgetItem(tree);
	//item->setText(tr("Vertical Split") );
	//item->setData(4, QVariant(-1) );
	//item=new QListWidgetItem(tree);
	//item->setText(tr("Horizontal Split") );
	//item->setData(4, QVariant(-2) );
	
	QGroupBox *box=new QGroupBox(this);
	QVBoxLayout *vbox_box=new QVBoxLayout;
	box->setLayout(vbox_box);
	split->addWidget(box);
	box->setTitle("Windows' Layout");
	box->show();
	
	widget=new QWidget(this);
	vbox_box->addWidget(widget);
	QVBoxLayout *vbox_widget=new QVBoxLayout;
	widget->setLayout(vbox_widget);
	widget->layout()->setMargin(0);
	widget->show();
	
	split_widget=new QSplitter(widget);
	vbox_widget->addWidget(split_widget);
	split_widget->addWidget(build_AddButton(this));
	split_widget->addWidget(build_AddButton(this));
	
	
	{
		QHBoxLayout *hbox=new QHBoxLayout;
		vbox->addLayout(hbox);
		
		vertical_split_ok=new QCheckBox(this);
		hbox->addWidget(vertical_split_ok);
		vertical_split_ok->setText( tr("Vertical split") );
		vertical_split_ok->show();
		
		connect(vertical_split_ok, SIGNAL(stateChanged ( int) ), this, SLOT(change_split_orientation(int)) );
		
		show_in_main_window_button=new QPushButton( tr("Show within main window"),this);
		hbox->addWidget(show_in_main_window_button);
		show_in_main_window_button->show();
		connect(show_in_main_window_button, SIGNAL(clicked () ), this, SLOT(show_in_main_window_cb()) );
		
		
		#if defined Q_OS_WIN32
			
		#else
			show_out_main_window_button=new QPushButton( tr("Show out of main window"),this);
			hbox->addWidget(show_out_main_window_button);
			show_out_main_window_button->show();
			connect(show_out_main_window_button, SIGNAL(clicked () ), this, SLOT(show_out_main_window_cb()) );
		#endif
		
	}
}

void WindowsLayout::change_split_orientation(int state)
{
	if(state==Qt::Checked)
		split_widget->setOrientation(Qt::Vertical);
	else
		split_widget->setOrientation(Qt::Horizontal);
}

void WindowsLayout::add_widget_cb(AddButton *button)
{
	if(tree->currentItem()==NULL) return;
	int item=tree->currentItem()->data(4).toInt();
	if(item>=0)
	{
		button->widget_contained=list.at( item );
		button->setText(button->widget_contained->windowTitle());
		button->setIcon(button->widget_contained->windowIcon() );
	}
}

void WindowsLayout::show_in_main_window_cb()
{
	widget->hide();
	widget->setParent(work_space);
	work_space->addSubWindow(widget);
	widget->setWindowTitle(title->text());
	widget->setWindowIcon( icon_button->icon()  );
	widget->show();
	emit show_widgets(work_space);
	destroy(true);
}

void WindowsLayout::show_out_main_window_cb()
{
	QList<QMdiSubWindow *> widgets_list=work_space->subWindowList();
	if(widgets_list.size()>1)
	{
		widget->setParent(NULL);
		widget->setWindowTitle(title->text());
		widget->setWindowIcon( icon_button->icon() );
		widget->show();
		emit show_widgets(work_space);
		destroy(true);
	}
	else
	{
		QMessageBox::critical(this, "Main Window Empty", "Main Window can not be empty");
	}
}

void WindowsLayout::select_icon_cb()
{
	QString s = QFileDialog::getOpenFileName(
                    this,
                    "Choose a file",
                    "/home",
                    "Images (*.png *.xpm *.jpg)");
	if( !s.isEmpty() ) icon_button->setIcon(QIcon(s));
}

void WindowsLayout::set_work_space(QMdiArea *work_space)
{
	this->work_space=work_space;
}

void WindowsLayout::set_window_list(QList<QMdiSubWindow *> list)
{
	this->list=list;
	for (int i = 0; i < list.size(); ++i)
	{
		QListWidgetItem *item=new QListWidgetItem(tree);
		item->setText(list.at(i)->windowTitle() );
		item->setIcon(list.at(i)->windowIcon() );
		//item->setData(1, QVariant(QVariant::UserType,list.at(i)) );
		item->setData(4, QVariant(i) );
	}
}
